@@ -170,11 +170,10 @@ from solfege import lessonfilegui
 from solfege import abstract
 
 
-from solfege import learningtree
+from solfege import frontpage
 from solfege import fpeditor
 from solfege.trainingsetdlg import TrainingSetDialog
 from solfege.practisesheetdlg import PractiseSheetDialog
-from solfege.helpbrowser import HelpBrowser, DocViewer
 from solfege import filesystem
 from solfege import statistics
 
@@ -297,29 +296,34 @@ class ExerciseView(SelectWinBase):
         self.g_searchentry.connect('activate', self.on_search)
         gu.bButton(self.g_searchbox, _("Search"), callback=self.on_search, expand=False)
         self.show_all()
-    def _display_data(self, page, display_only_tests=False):
-        self._page_tree = page
+    def _display_data(self, page, display_only_tests=False,
+            is_search_result=False):
+        if not is_search_result:
+            self.m_page = page
         try:
             self.g_hbox.destroy()
         except AttributeError:
             pass
         self.g_hbox = gu.bHBox(self.g_box)
+        self.g_hbox.set_spacing(gu.hig.SPACE_MEDIUM)
+        label = None
         for column in page:
-            assert isinstance(column, learningtree.Column)
+            sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+            assert isinstance(column, frontpage.Column)
             first = True
             vbox = gtk.VBox()
             vbox.set_spacing(gu.hig.SPACE_MEDIUM)
             self.g_hbox.pack_start(vbox, False)
             for sect_id, linklist in enumerate(column):
-                if isinstance(linklist, learningtree.Paragraph):
+                if isinstance(linklist, frontpage.Paragraph):
                     label = gtk.Label(linklist.m_text)
                     label.set_line_wrap(True)
                     label.set_alignment(0.0, 0.5)
                     vbox.pack_start(label, False)
                     continue
-                assert isinstance(linklist, learningtree.LinkList)
+                assert isinstance(linklist, frontpage.LinkList)
                 if (display_only_tests
-                    and not learningtree._TreeCommon.tests_in_sub(linklist)):
+                    and not frontpage._TreeCommon.tests_in_sub(linklist)):
                         continue
                 linkbox = gtk.VBox()
                 vbox.pack_start(linkbox, False)
@@ -328,52 +332,50 @@ class ExerciseView(SelectWinBase):
                 heading.set_use_markup(True)
                 linkbox.pack_start(heading, False)
                 for idx, link in enumerate(linklist):
-                    if isinstance(self, TestsView) and not learningtree._TreeCommon.tests_in_sub(link):
+                    if isinstance(self, TestsView) and not frontpage._TreeCommon.tests_in_sub(link):
                         continue
-                    if isinstance(link, learningtree.Page):
+                    if isinstance(link, frontpage.Page):
                         label = gu.ClickableLabel(_no_xgettext(link.m_name))
                         label.connect('clicked', self.on_page_link_clicked, link)
                     else:
-                        assert isinstance(link, unicode)
-                        # The warning sign will be showed if no lesson file
-                        # with the lesson_id is found by lessonfile.manager.
-                        #
-                        # It is possible that the lesson_id exists in the
-                        # statistics database even though it is not available
-                        # on file, if it has been practised earlier on and
-                        # then delete.
-                        link_ok = True
+                        assert isinstance(link, unicode), type(link)
+                        solfege.db.validate_stored_statistics(link)
                         try:
-                            labeltxt = solfege.db.get_field(link, "title")
+                            labeltxt = lessonfile.infocache.get(link, 'title')
                             label = gu.ClickableLabel(_no_xgettext(labeltxt))
-                            if link not in solfege.lessonfile.manager.m_uiddb:
-                                link_ok = False
-                                label.set_tooltip_text(_("The 'lesson_id' of the missing file is\n'%s'. The previously known file is not available now") % link)
-                            if link_ok:
-                                label.connect('clicked', self.on_link_clicked, link)
-                        except KeyError:
-                            labeltxt = _("Lesson file not found")
-                            label = gu.ClickableLabel(_no_xgettext(labeltxt))
-                            label.set_tooltip_text(_("The lesson id of the missing file is\n'%s'") % link)
-                            link_ok = False
-                        if not link_ok:
-                            im = gtk.Image()
-                            im.set_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_MENU)
-                            label.set_image(im)
-                            label.set_alignment(0.0, 0.5)
+                            label.connect('clicked', self.on_link_clicked, link)
+                        except lessonfile.InfoCache.FileNotFound:
+                            label = gu.ClickableLabel(_(u"«%s» was not found") % link)
+                            label.make_warning()
+                        except lessonfile.InfoCache.FileNotLessonfile:
+                            label = gu.ClickableLabel(_(u"Failed to parse «%s»") % link)
+                            label.make_warning()
                     if first:
                         label.set_data('first', True)
                         first = False
                     w = label.size_request()[0]
                     if w > self.max_exercise_label_width:
                         w = self.max_exercise_label_width
-                        txt = _(solfege.db.get_field(link, "title"))
+                        txt = _(lessonfile.infocache.get(link, "title"))
                         label.set_tooltip_text(txt)
                         label.set_size_request(w, -1)
                     self.m_linkbuttons.append(label)
                     label.connect('focus-in-event', self.on_focus_in)
-                    linkbox.pack_start(label, True)
-            if not page.is_empty():
+                    if display_only_tests:
+                        box = gtk.HBox()
+                        box.pack_start(label)
+                        if isinstance(link, unicode):
+                            passed, result = solfege.db.get_test_status(link)
+                            if passed == True:
+                                box.pack_start(gtk.Label(_("passed, %.1f%%") % (result * 100)))
+                            if passed == False:
+                                box.pack_start(gtk.Label(_("failed, %.1f%%") % (result * 100)))
+                            # do nothing if passed == None since 
+                        linkbox.pack_start(box, True)
+                    else:
+                        linkbox.pack_start(label, True)
+                    sizegroup.add_widget(label)
+            if label:
                 label.set_data('last', True)
         self.g_hbox.show_all()
         self.adjust_scrolledwin_size()
@@ -381,17 +383,17 @@ class ExerciseView(SelectWinBase):
     def on_page_link_clicked(self, btn, link):
         self.g_searchbox.hide()
         self.display_data(link)
-    def on_link_clicked(self, w, uuid):
-        solfege.app.practise_lesson_id(uuid)
+    def on_link_clicked(self, w, filename):
+        solfege.app.practise_lessonfile(filename)
     def display_search_result(self, searchfor, result, result_C, display_only_tests=False):
         self._display_data(
-          learningtree.Page(u'',
-            learningtree.Column([
-              learningtree.LinkList(
-            _("Search results for '%s':" % self.g_searchentry.get_text()),
+          frontpage.Page(u'',
+            frontpage.Column([
+              frontpage.LinkList(
+            _("Search results for “%s”:" % self.g_searchentry.get_text()),
               result),
-              learningtree.LinkList(
-            _("C-locale search results for '%s':" % self.g_searchentry.get_text()),
+              frontpage.LinkList(
+            _("C-locale search results for “%s”:" % self.g_searchentry.get_text()),
               result_C),
               ])))
     def on_end_practise(self, w=None):
@@ -403,53 +405,59 @@ class ExerciseView(SelectWinBase):
         only_tests - True if we should only return exercises that have
                      tests
         """
+        logging.debug("search: '%s'" % substring)
         match_func = {
-            False: lambda lesson_id: substring in _no_xgettext(solfege.lessonfile.manager.get(lesson_id, 'title')).lower(),
-            True: lambda lesson_id: substring in solfege.lessonfile.manager.get(lesson_id, 'title').lower()
+            False: lambda filename: substring in _no_xgettext(lessonfile.infocache.get(filename, 'title')).lower(),
+            True: lambda filename: substring in lessonfile.infocache.get(filename, 'title').lower()
         }[C_locale]
         test_filter = {
-            False: lambda lesson_id: True,
-            True: lambda lesson_id: solfege.lessonfile.manager.get(lesson_id, 'test')
+            False: lambda filename: True,
+            True: lambda filename: lessonfile.infocache.get(filename, 'test')
         }[C_locale]
-        page = learningtree.Page(listitems=learningtree.Column())
+        page = frontpage.Page(listitems=frontpage.Column())
         cur_topic = None
         # the last topic appended to the page
         last_topic = None 
-        for child in solfege.app.m_frontpage_data.iterate_flattened():
-            if isinstance(child, learningtree.LinkList):
+        found = set()
+        for child in self.m_page.iterate_flattened():
+            if isinstance(child, frontpage.LinkList):
                 cur_topic = child
             if isinstance(child, unicode):
-                if match_func(child) and test_filter(child):
-                    if cur_topic != last_topic:
-                        page[0].append(learningtree.LinkList(cur_topic.m_name))
-                        last_topic = cur_topic
-                    page[0][-1].append(child)
+                try:
+                    if (match_func(child) and test_filter(child)) and child not in found:
+                        if cur_topic != last_topic:
+                            page[0].append(frontpage.LinkList(cur_topic.m_name))
+                            last_topic = cur_topic
+                        found.add(child)
+                        page[0][-1].append(child)
+                except lessonfile.infocache.InfoCacheException:
+                    # Ignore missing lesson files and files with errors
+                    pass
         return page
 
 
 class FrontPage(ExerciseView):
-    def display_data(self, data=None, display_only_tests=False):
-        if not data:
-            data = solfege.app.m_frontpage_data
-        self._display_data(data, display_only_tests)
+    def display_data(self, data, display_only_tests=False,
+            is_search_result=False):
+        self._display_data(data, display_only_tests, is_search_result)
         self.g_searchentry.grab_focus()
     def on_search(self, *button):
-        solfege.win.wait_until_startup_is_complete()
         search_for = self.g_searchentry.get_text().lower()
+        logging.debug("FrontPage.on_search '%s'" % search_for)
         page = self._search(search_for, False, False)
         page_C = self._search(search_for, True, False)
-        page_C.m_name = u'Search exercises without translating them'
+        page_C.m_name = _('Search untranslated lesson titles')
         if not page_C.is_empty():
-            page[0].append(learningtree.LinkList(_('Too few matches?'),
+            page[0].append(frontpage.LinkList(_('Too few matches?'),
                 listitems=[page_C]))
-        self.display_data(page)
+        self.display_data(page, is_search_result=True)
 
 
 class TestsView(ExerciseView):
-    def on_link_clicked(self, w, uuid):
-        solfege.app.test_lesson_id(uuid)
-    def display_data(self, data=None):
-        self._display_data(data, True)
+    def on_link_clicked(self, w, filename):
+        solfege.app.test_lessonfile(filename)
+    def display_data(self, data=None, is_search_result=False):
+        self._display_data(data, True, is_search_result)
         self.g_searchentry.grab_focus()
     def on_search(self, *button):
         search_for = self.g_searchentry.get_text().lower()
@@ -457,43 +465,43 @@ class TestsView(ExerciseView):
         page_C = self._search(search_for, True, True)
         page_C.m_name = u'Search exercises without translating them'
         if not page_C.is_empty():
-            page[0].append(learningtree.LinkList(_('Too few matches?'),
+            page[0].append(frontpage.LinkList(_('Too few matches?'),
                 listitems=[page_C]))
-        self.display_data(page)
+        self.display_data(page, is_search_result=True)
 
 
 class SearchView(ExerciseView):
     def __init__(self):
         ExerciseView.__init__(self)
-        page = """Page(_(u'xDefault front page'), [
+        page = """FileHeader(1, 
+        Page(u'', [
          Column([
           Paragraph(u'Search the exercise titles of all lesson files found by the program, not just the active front page with sub pages.'),
          ]),
-        ])"""
-        self.display_data(learningtree.parse_tree(page))
+        ])
+        )"""
+        p = frontpage.parse_tree(page)
+        self.display_data(frontpage.parse_tree(page))
         self.g_searchentry.show()
     def display_data(self, data):
         self._display_data(data)
         self.g_searchentry.grab_focus()
     def on_search(self, *button):
         search_for = self.g_searchentry.get_text().lower()
+        lessonfile.infocache.update_modified_files()
         self.display_search_result(self.g_searchentry.get_text(), [
-            lesson_id 
-            for lesson_id in solfege.lessonfile.manager.iterate_lesson_ids()
-            if search_for in _no_xgettext(solfege.lessonfile.manager.get(
-                lesson_id, 'title')).lower()
+            filename for filename in lessonfile.infocache._data
+            if search_for in _(lessonfile.infocache.get(filename, 'title')).lower()
         ], [
-            lesson_id 
-            for lesson_id in solfege.lessonfile.manager.iterate_lesson_ids()
-            if search_for in solfege.lessonfile.manager.get(
-                lesson_id, 'title').lower()
+            filename for filename in lessonfile.infocache._data
+            if search_for in lessonfile.infocache.get(filename, 'title').lower()
         ])
     
 
 class MainWin(gtk.Window, cfg.ConfigUtils):
-    default_learning_tree = ('solfege', 'learningtree.txt')
-    debug_learning_tree = ('solfege', 'debugtree.txt')
-    def __init__(self, options, datadir, lessonfile_manager):
+    default_front_page = os.path.join(lessonfile.exercises_dir, 'learningtree.txt')
+    debug_front_page = os.path.join(lessonfile.exercises_dir, 'debugtree.txt')
+    def __init__(self, options, datadir):
         gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
         self._vbox = gtk.VBox()
         self._vbox.show()
@@ -501,14 +509,13 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         stock.SolfegeIconFactory(self, datadir)
         gtk.settings_get_default().set_property('gtk-button-images', True)
         pixbuf = self.render_icon('solfege-icon', gtk.ICON_SIZE_DIALOG)
-        self.set_icon(pixbuf)
+        gtk.window_set_default_icon(pixbuf)
         cfg.ConfigUtils.__dict__['__init__'](self, 'mainwin')
         self.set_resizable(self.get_bool('gui/mainwin_user_resizeable'))
         self.add_watch('gui/mainwin_user_resizeable', lambda s: self.set_resizable(self.get_bool('gui/mainwin_user_resizeable')))
         self.connect('delete-event', self.quit_program)
         self.connect('key_press_event', self.on_key_press_event)
         self.g_about_window = None
-        self.g_help_browser = None
         self.m_exercise = None
         self.m_viewer = None
         self.box_dict = {}
@@ -526,7 +533,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         self.main_box = gtk.VBox()
         self.main_box.show()
         self._vbox.pack_start(self.main_box)
-        self.on_learning_tree_changed()
+        self.on_frontpage_changed()
         self.display_frontpage()
     def get_view(self):
         """
@@ -558,39 +565,31 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         self.m_viewer = name
         self.box_dict[name].show()
         return True
-    def change_learning_tree(self, tree_fn):
+    def change_frontpage(self, filename):
         """
-        Change to a different learning tree.
+        Change to a different frontpage file.
         """
-        self.set_list('app/learningtree', tree_fn)
-        self.on_learning_tree_changed()
-    def on_learning_tree_changed(self, *v):
+        self.set_string('app/frontpage', filename)
+        self.on_frontpage_changed()
+    def on_frontpage_changed(self, *v):
         """
         Load the frontpage data from the file specified by the config
-        variable "app/learningtree"
+        variable "app/frontpage"
         """
+        filename = self.get_string("app/frontpage")
+        if filename == self.debug_front_page and not solfege.app.m_options.debug:
+            self.set_string("app/frontpage", self.default_front_page)
+            filename = self.default_front_page
+        if not os.path.isfile(filename):
+            filename = self.default_front_page
         try:
-            loc, filename = self.get_list("app/learningtree")
-        except ValueError, e:
-            logging.debug("cfg var app/learningtree did not unpack as a 2 elem tuple: '%s'" % self.get_string("app/learningtree"))
-            loc, filename = self.default_learning_tree
-        if (loc, filename) == ('solfege', 'debugtree.txt') and not solfege.app.m_options.debug:
-            self.set_list("app/learningtree", ('solfege', 'learningtree.txt'))
-            loc, filename = self.default_learning_tree
-        if loc == 'solfege':
-            tree_filename = os.path.join("learningtrees", filename)
-        else:
-            assert loc == 'user'
-            tree_filename = os.path.join(filesystem.user_data(), 'learningtrees', filename)
-        try:
-            solfege.app.m_frontpage_data = learningtree.load_tree(tree_filename)
-        except IOError, e:
+            solfege.app.m_frontpage_data = frontpage.load_tree(filename)
+        except Exception, e:
             if splash_win:
                 splash_win.hide()
-            solfege.app.m_frontpage_data = learningtree.load_tree(
-                os.path.join("learningtrees", self.default_learning_tree[1]))
-            self.set_list('app/learningtree', self.default_learning_tree)
-            gu.dialog_ok(_("Learning tree '%s' not found. Using default tree." % filename),
+            solfege.app.m_frontpage_data = frontpage.load_tree(self.default_front_page)
+            self.set_string('app/frontpage', self.default_front_page)
+            gu.dialog_ok(_("Learning front page '%s' failed. Using default page." % filename),
                 secondary_text = "\n".join(traceback.format_exception(*sys.exc_info())))
             if splash_win:
                 splash_win.show()
@@ -622,6 +621,8 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
               self.display_testpage),
           ('RecentExercises', None, _('_Recent Exercises'), 'F7', None,
               self.display_recent_exercises),
+          ('UserExercises', None, _('_User Exercises'), 'F8', None,
+              self.display_user_exercises),
           ('HelpHelp', 'gtk-help', _('_Help on the current exercise'), 'F1', None,
             lambda o: solfege.app.please_help_me()),
           ('HelpTheory', None, _('_Music theory on the current exercise'), 'F3', None, lambda o: solfege.app.show_exercise_theory()),
@@ -646,41 +647,33 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         hdlbox.add(self.g_ui_manager.get_widget('/Menubar'))
         self._vbox.pack_start(hdlbox, False)
         self.m_help_on_current_merge_id = None
-    def create_learning_trees_menu(self):
-        if self.m_learning_trees_merge_id:
-            self.g_ui_manager.remove_ui(self.m_learning_trees_merge_id)
-        trees = learningtree.get_learning_tree_list(solfege.app.m_options.debug)
+    def create_frontpage_menu(self):
+        """
+        Create, or update if already existing, the submenu that let the
+        user choose which front page file to display.
+        """
+        if self.m_frontpage_merge_id:
+            self.g_ui_manager.remove_ui(self.m_frontpage_merge_id)
         actions = []
-        s = """
-<menubar name='Menubar'>
- <menu action='FileMenu'>
-  <menu action='FrontPagesMenu'>\n"""
-        old_cls = trees[0][0]
-        for cls, fn, fullpath in trees:
-            # FIXME: the separator seems to be invisible most of the time. Why???
-            if old_cls != cls:
-                s += "<separator/>\n"
-                old_cls = cls
-            try:
-                t = learningtree.load_tree(fullpath)
-                assert isinstance(t, learningtree.Page)
-            except Exception, e:
-                if splash_win:
-                    splash_win.hide()
-                gu.dialog_ok(_("Failed to load learning tree"), self, 
-                    _("Ignoring the file \"%(filename)s\" because of this error: %(error)s") % {
-                    'filename': fullpath,
-                    'error': str(e).decode(sys.getfilesystemencoding(), 'replace'),
-                    }, gtk.MESSAGE_ERROR)
+        old_dir = None
+        s = "<menubar name='Menubar'><menu action='FileMenu'><menu action='FrontPagesMenu'>"
+        for fn in frontpage.get_front_pages_list(solfege.app.m_options.debug):
+            if not frontpage.may_be_frontpage(fn):
                 continue
-            s += "  <menuitem action='%s:%s'/>\n" % (cls, fn)
-            action_name = '%s:%s' % (cls, fn)
-            if action_name not in [x.get_name() for x in self.m_action_groups['NotExit'].list_actions()]:
-                actions.append((action_name, None, t.m_name, None, None,
-                lambda o, c=cls, f=fn: self.change_learning_tree((c, f))))
+            cur_dir = os.path.split(fn)[0]
+            if old_dir != cur_dir:
+                s += '<separator name="sep@%s"/>' % fn
+                old_dir = cur_dir
+            s += "<menuitem action='%s'/>\n" % fn
+            if not self.m_action_groups['NotExit'].get_action(fn):
+                actions.append((fn, None, lessonfile.infocache.frontpage.get(fn, 'title'), None, fn,
+                lambda o, f=fn: self.change_frontpage(f)))
+            else:
+                action = self.m_action_groups['NotExit'].get_action(fn)
+                action.props.label = lessonfile.infocache.frontpage.get(fn, 'title')
         s += "</menu></menu></menubar>"
         self.m_action_groups['NotExit'].add_actions(actions)
-        self.m_learning_trees_merge_id = self.g_ui_manager.add_ui_from_string(s)
+        self.m_frontpage_merge_id = self.g_ui_manager.add_ui_from_string(s)
     def show_help_on_current(self):
         """
         Show the menu entries for the exercise help and music theory
@@ -801,10 +794,13 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
             box1, box2 = gu.hig_category_vbox(_("File locations"))
             vbox.pack_start(box1)
             sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+            # statistics.sqlite
+            # win32 solfegerc
+            # win32 langenviron.txt
+            box2.pack_start(gu.hig_label_widget(_("Solfege application data:"), gtk.Label(filesystem.app_data()), sizegroup))
+            box2.pack_start(gu.hig_label_widget(_("Solfege user data:"), gtk.Label(filesystem.user_data()), sizegroup))
             box2.pack_start(gu.hig_label_widget(_("Solfege config file:"), gtk.Label(filesystem.rcfile()), sizegroup))
-            box2.pack_start(gu.hig_label_widget(_("Solfege standard lesson files:"), gtk.Label(os.path.join(os.getcwdu(), "lesson-files")), sizegroup))
-            box2.pack_start(gu.hig_label_widget(_("The users private lesson files:"), gtk.Label(filesystem.user_lessonfiles()), sizegroup))
-            box2.pack_start(gu.hig_label_widget(_("The users private learning trees:"), gtk.Label(os.path.join(filesystem.user_data(), "learningtrees")), sizegroup))
+            box2.pack_start(gu.hig_label_widget(_("Solfege installation directory:"), gtk.Label(os.getcwdu()), sizegroup))
             box2.pack_start(gu.hig_label_widget(_("User manual in HTML format:"), gtk.Label(os.path.join(os.getcwdu(), "help")), sizegroup))
             self.g_path_info_dlg.show_all()
             def f(*w):
@@ -847,16 +843,11 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         """
         Open a learning tree editor editing the current front page.
         """
-        try:
-            loc, filename = self.get_list("app/learningtree")
-        except ValueError:
-            loc, filename = self.default_learning_tree
-        fn = fpeditor.Editor.tuple_to_filename(loc, filename)
-        fpeditor.Editor.edit_file(fn)
+        fpeditor.Editor.edit_file(self.get_string("app/frontpage"))
     def post_constructor(self):
         global splash_win
-        self.m_learning_trees_merge_id = None
-        self.create_learning_trees_menu()
+        self.m_frontpage_merge_id = None
+        self.create_frontpage_menu()
         self.g_ui_manager.add_ui_from_file("help-menu.xml")
         if solfege.app.m_sound_init_exception is not None:
             if splash_win:
@@ -891,19 +882,13 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         if sys.platform == 'win32' and winlang.win32_get_langenviron() != self.get_string('app/lc_messages'):
             gu.dialog_ok(_("Migrated old language setup. You might have to restart the program all translated messages to show up."))
             winlang.win32_put_langenviron(self.get_string('app/lc_messages'))
-        if solfege.app.lessonfile_manager.m_discards:
-            if splash_win:
-                splash_win.hide()
-            self.display_discarded_lessonfiles_info()
-            if splash_win:
-                splash_win.show()
         # MIGRATION 3.11.1: earlier editors would create new learning trees
         # below app_data() instead of user_data().
         if (sys.platform == "win32" and 
             os.path.exists(os.path.join(filesystem.app_data(),
                                         "learningtrees"))):
             if not os.path.exists(os.path.join(filesystem.user_data(), "learningtrees")):
-                os.mkdir(os.path.join(filesystem.user_data(), "learningtrees"))
+                os.makedirs(os.path.join(filesystem.user_data(), "learningtrees"))
             for fn in os.listdir(os.path.join(filesystem.app_data(), "learningtrees")):
                 if not os.path.exists(os.path.join(filesystem.user_data(), "learningtrees", fn)):
                     shutil.move(os.path.join(filesystem.app_data(), "learningtrees", fn),
@@ -914,7 +899,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
                             os.path.join(filesystem.user_data(), "learningtrees", u"%s.bak" % fn))
                 os.rmdir(os.path.join(os.path.join(filesystem.app_data(), "learningtrees")))
         item = self.g_ui_manager.get_widget("/Menubar/FileMenu/FrontPagesMenu")
-        item.connect('activate', lambda s: self.create_learning_trees_menu())
+        item.connect('activate', lambda s: self.create_frontpage_menu())
         try:
             i18n.locale_setup_failed
             print >> sys.stderr, "\n".join(textwrap.wrap("Translations are disabled because your locale settings are broken. This is not a bug in GNU Solfege, so don't report it. The README file distributed with the program has some more details."))
@@ -983,44 +968,48 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         """
         Display the HTML file named by fn in the help browser window.
         """
-        if self.get_bool('gui/web_browser_as_help_browser'):
-            for lang in solfege.app.m_userman_language, "C":
-                filename = os.path.join(os.getcwdu(), u"help", lang, fn)
-                if os.path.isfile(filename):
-                    break
-            try:
-                webbrowser.open(filename)
-            except Exception, e:
-                self.display_error_message2(_("Error opening web browser"), str(e))
-        else:
-            if not self.g_help_browser:
-                self.g_help_browser = HelpBrowser()
-            self.g_help_browser.show()
-            self.g_help_browser.present()
-            self.g_help_browser.show_docfile(fn, anchor)
-    def wait_until_startup_is_complete(self):
-        if not solfege.app.m_startup_complete:
-            dlg = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO,
-                0, _("Please wait"))
-            dlg.show_all()
-            while not solfege.app.m_startup_complete:
-                while gtk.events_pending():
-                    gtk.main_iteration(0)
-                time.sleep(0.1)
-            dlg.destroy()
+        for lang in solfege.app.m_userman_language, "C":
+            filename = os.path.join(os.getcwdu(), u"help", lang, fn)
+            if os.path.isfile(filename):
+                break
+        try:
+            webbrowser.open(filename)
+        except Exception, e:
+            self.display_error_message2(_("Error opening web browser"), str(e))
+    def display_user_exercises(self, w):
+        col = frontpage.Column()
+        page = frontpage.Page(_('User exercises'), col)
+        curdir =None
+        for filename in lessonfile.infocache.iter_user_files():
+            dir, fn = os.path.split(filename)
+            if dir != curdir:
+                curdir = dir
+                linklist = frontpage.LinkList(dir)
+                col.append(linklist)
+            linklist.append(filename)
+        if os.path.isdir(filesystem.user_lessonfiles()):
+            linklist = None
+            # Added just to be nice with people not moving their files from
+            # pre 3.15.3 location:
+            for filename in os.listdir(filesystem.user_lessonfiles()):
+                if not linklist:
+                    linklist = frontpage.LinkList(filesystem.user_lessonfiles())
+                linklist.append(os.path.join(filesystem.user_lessonfiles(), filename))
+            # only display the linklist if there are any files.
+            if linklist:
+                col.append(linklist)
+        self.display_frontpage(page)
     def display_recent_exercises(self, w):
-        data = learningtree.Page(_('Recent exercises'),
-            [learningtree.Column(
-                [learningtree.LinkList(_('Recent exercises'),
+        data = frontpage.Page(_('Recent exercises'),
+            [frontpage.Column(
+                [frontpage.LinkList(_('Recent exercises'),
                    solfege.db.recent(8))])])
         self.display_frontpage(data)
+        self.get_view().g_searchbox.hide()
     def display_testpage(self, widget=None):
         """
         Display the front page of the data  in solfege.app.m_frontpage_data
         """
-        # We cannot handle clicks to the links in the TestsView before
-        # the lessonfile manager has completed parsing lesson files.
-        self.wait_until_startup_is_complete()
         self.set_title("GNU Solfege - tests")
         if not self.show_view('testspage'):
             self.add_view(TestsView(), 'testspage')
@@ -1038,6 +1027,8 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         if not self.show_view('frontpage'):
             self.add_view(FrontPage(), 'frontpage')
         self.get_view().g_searchbox.show()
+        if not data:
+            data = solfege.app.m_frontpage_data
         self.get_view().display_data(data)
     def initialise_exercise(self, teacher):
         """
@@ -1102,8 +1093,6 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
     def new_practisesheet_editor(self, widget):
         dlg = PractiseSheetDialog()
         dlg.show_all()
-    def handle_lesson_id_crash(self):
-        lessonfilegui.handle_lesson_id_crash(lessonfile.manager)
 
 import locale
 locale.setlocale(locale.LC_NUMERIC, "C")
@@ -1116,13 +1105,14 @@ cfg.set_bool('config/no_random', bool(options.no_random))
 
 if splash_win:
     splash_win.show_progress("creating LessonfileManager")
-lessonfile_manager = lessonfile.LessonFileManager()
-lessonfile.manager = lessonfile_manager
+lessonfile.infocache = lessonfile.InfoCache()
+
 if splash_win:
     if not os.path.exists(statistics.DB.get_statistics_filename()):
         splash_win.show_progress("Importing old statistics:")
 def f(s):
-    splash_win.show_progress("Importing old statistics: %i%%" % s)
+    if splash_win:
+        splash_win.show_progress(s)
 solfege.db = statistics.DB(f)
 
 if splash_win:
@@ -1130,8 +1120,8 @@ if splash_win:
 
 def start_app(datadir):
     global splash_win
-    solfege.app = application.SolfegeApp(options, lessonfile_manager)
-    solfege.win = w = MainWin(options, datadir, lessonfile_manager)
+    solfege.app = application.SolfegeApp(options)
+    solfege.win = w = MainWin(options, datadir)
     solfege.app.setup_sound()
     w.show()
     w.post_constructor()
@@ -1156,5 +1146,9 @@ def start_app(datadir):
     if not options.disable_exception_handler:
         sys.excepthook = ef
     print time.time() - start_time
+    # We parse all lesson files when we are idle to save a half a
+    # second the first time the user searches all lesson files using
+    # Ctrl-F.
+    lessonfile.infocache.parse_all_files(True)
     gtk.main()
 
